@@ -1,4 +1,6 @@
 /*
+ * SPDX-License-Identifier: ISC
+ *
  * Copyright (c) 2013-2015 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -14,26 +16,18 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+/*
+ * This is an open source non-commercial project. Dear PVS-Studio, please check it.
+ * PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+ */
+
 #include <config.h>
 
-#include <sys/types.h>
-#include <stdio.h>
+#include <sys/resource.h>
+
 #include <stdlib.h>
-#ifdef HAVE_STDBOOL_H
-# include <stdbool.h>
-#else
-# include "compat/stdbool.h"
-#endif /* HAVE_STDBOOL_H */
-#ifdef HAVE_STRING_H
-# include <string.h>
-#endif /* HAVE_STRING_H */
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif /* HAVE_STRINGS_H */
-#include <time.h>
-#include <unistd.h>
-#include <errno.h>
 #include <poll.h>
+#include <time.h>
 
 #include "sudo_compat.h"
 #include "sudo_util.h"
@@ -45,7 +39,7 @@ int
 sudo_ev_base_alloc_impl(struct sudo_event_base *base)
 {
     int i;
-    debug_decl(sudo_ev_base_alloc_impl, SUDO_DEBUG_EVENT)
+    debug_decl(sudo_ev_base_alloc_impl, SUDO_DEBUG_EVENT);
 
     base->pfd_high = -1;
     base->pfd_max = 32;
@@ -66,7 +60,7 @@ sudo_ev_base_alloc_impl(struct sudo_event_base *base)
 void
 sudo_ev_base_free_impl(struct sudo_event_base *base)
 {
-    debug_decl(sudo_ev_base_free_impl, SUDO_DEBUG_EVENT)
+    debug_decl(sudo_ev_base_free_impl, SUDO_DEBUG_EVENT);
     free(base->pfds);
     debug_return;
 }
@@ -74,29 +68,49 @@ sudo_ev_base_free_impl(struct sudo_event_base *base)
 int
 sudo_ev_add_impl(struct sudo_event_base *base, struct sudo_event *ev)
 {
+    static int nofile_max = -1;
     struct pollfd *pfd;
-    debug_decl(sudo_ev_add_impl, SUDO_DEBUG_EVENT)
+    debug_decl(sudo_ev_add_impl, SUDO_DEBUG_EVENT);
+
+    if (nofile_max == -1) {
+	struct rlimit rlim;
+	if (getrlimit(RLIMIT_NOFILE, &rlim) == 0) {
+	    nofile_max = rlim.rlim_cur;
+	}
+    }
 
     /* If out of space in pfds array, realloc. */
     if (base->pfd_free == base->pfd_max) {
 	struct pollfd *pfds;
-	int i;
+	int i, new_max;
 
-	pfds =
-	    reallocarray(base->pfds, base->pfd_max, 2 * sizeof(struct pollfd));
+	/* Don't allow pfd_max to go over RLIM_NOFILE */
+	new_max = base->pfd_max * 2;
+	if (new_max > nofile_max)
+	    new_max = nofile_max;
+	if (base->pfd_free == new_max) {
+	    sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
+		"%s: out of fds (max %d)", __func__, nofile_max);
+	    debug_return_int(-1);
+	}
+	sudo_debug_printf(SUDO_DEBUG_INFO|SUDO_DEBUG_LINENO,
+	    "%s: pfd_max %d -> %d", __func__, base->pfd_max, new_max);
+	pfds = reallocarray(base->pfds, new_max, sizeof(struct pollfd));
 	if (pfds == NULL) {
 	    sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO,
-		"%s: unable to allocate %d pollfds", __func__, base->pfd_max * 2);
+		"%s: unable to allocate %d pollfds", __func__, new_max);
 	    debug_return_int(-1);
 	}
 	base->pfds = pfds;
-	base->pfd_max *= 2;
+	base->pfd_max = new_max;
 	for (i = base->pfd_free; i < base->pfd_max; i++) {
 	    base->pfds[i].fd = -1;
 	}
     }
 
     /* Fill in pfd entry. */
+    sudo_debug_printf(SUDO_DEBUG_DEBUG|SUDO_DEBUG_LINENO,
+	"%s: choosing free slot %d", __func__, base->pfd_free);
     ev->pfd_idx = base->pfd_free;
     pfd = &base->pfds[ev->pfd_idx];
     pfd->fd = ev->fd;
@@ -122,12 +136,15 @@ sudo_ev_add_impl(struct sudo_event_base *base, struct sudo_event *ev)
 int
 sudo_ev_del_impl(struct sudo_event_base *base, struct sudo_event *ev)
 {
-    debug_decl(sudo_ev_del_impl, SUDO_DEBUG_EVENT)
+    debug_decl(sudo_ev_del_impl, SUDO_DEBUG_EVENT);
 
     /* Mark pfd entry unused, add to free list and adjust high slot. */
     base->pfds[ev->pfd_idx].fd = -1;
-    if (ev->pfd_idx < base->pfd_free)
+    if (ev->pfd_idx < base->pfd_free) {
 	base->pfd_free = ev->pfd_idx;
+	sudo_debug_printf(SUDO_DEBUG_DEBUG|SUDO_DEBUG_LINENO,
+	    "%s: new free slot %d", __func__, base->pfd_free);
+    }
     while (base->pfd_high >= 0 && base->pfds[base->pfd_high].fd == -1)
 	base->pfd_high--;
 
@@ -144,7 +161,8 @@ sudo_ev_poll(struct pollfd *fds, nfds_t nfds, const struct timespec *timo)
 static int
 sudo_ev_poll(struct pollfd *fds, nfds_t nfds, const struct timespec *timo)
 {
-    const int timeout = (timo->tv_sec * 1000) + (timo->tv_nsec / 1000000);
+    const int timeout =
+	timo ? (timo->tv_sec * 1000) + (timo->tv_nsec / 1000000) : -1;
 
     return poll(fds, nfds, timeout);
 }
@@ -156,12 +174,12 @@ sudo_ev_scan_impl(struct sudo_event_base *base, int flags)
     struct timespec now, ts, *timeout;
     struct sudo_event *ev;
     int nready;
-    debug_decl(sudo_ev_scan_impl, SUDO_DEBUG_EVENT)
+    debug_decl(sudo_ev_scan_impl, SUDO_DEBUG_EVENT);
 
     if ((ev = TAILQ_FIRST(&base->timeouts)) != NULL) {
 	sudo_gettime_mono(&now);
 	sudo_timespecsub(&ev->timeout, &now, &ts);
-	if (ts.tv_sec < 0 || (ts.tv_sec == 0 && ts.tv_nsec < 0))
+	if (ts.tv_sec < 0)
 	    sudo_timespecclear(&ts);
 	timeout = &ts;
     } else {
@@ -174,16 +192,20 @@ sudo_ev_scan_impl(struct sudo_event_base *base, int flags)
     }
 
     nready = sudo_ev_poll(base->pfds, base->pfd_high + 1, timeout);
-    sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %d fds ready", __func__, nready);
     switch (nready) {
     case -1:
-	/* Error or interrupted by signal. */
-	debug_return_int(-1);
+	/* Error: EINTR (signal) or EINVAL (nfds > RLIMIT_NOFILE) */
+	sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_LINENO|SUDO_DEBUG_ERRNO,
+	    "sudo_ev_poll");
+	break;
     case 0:
 	/* Front end will activate timeout events. */
+	sudo_debug_printf(SUDO_DEBUG_INFO, "%s: timeout", __func__);
 	break;
     default:
 	/* Activate each I/O event that fired. */
+	sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %d fds ready", __func__,
+	    nready);
 	TAILQ_FOREACH(ev, &base->events, entries) {
 	    if (ev->pfd_idx != -1 && base->pfds[ev->pfd_idx].revents) {
 		int what = 0;
