@@ -76,7 +76,7 @@ inet_ntop4(const unsigned char *src, char *dst, socklen_t size)
 	int len;
 
 	len = snprintf(dst, size, fmt, src[0], src[1], src[2], src[3]);
-	if (len < 0 || len >= size) {
+	if (len < 0 || (size_t)len >= size) {
 		errno = ENOSPC;
 		return (NULL);
 	}
