@@ -118,10 +118,7 @@ init(void)
     VERIFY_TRUE(asprintf(&data.tmp_dir, TEMP_PATH_TEMPLATE) >= 0);
     VERIFY_NOT_NULL(mkdtemp(data.tmp_dir));
 
-    // by default we test in developer mode, so the python plugin can be loaded
     sudo_conf_clear_paths();
-    VERIFY_INT(sudo_conf_read(sudo_conf_developer_mode, SUDO_CONF_ALL), true);
-    VERIFY_TRUE(sudo_conf_developer_mode());
 
     // some default values for the plugin open:
     data.settings = create_str_array(1, NULL);
@@ -464,6 +461,7 @@ check_example_group_plugin(void)
     return true;
 }
 
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 static const char *
 create_debug_config(const char *debug_spec)
 {
@@ -473,8 +471,7 @@ create_debug_config(const char *debug_spec)
     snprintf(config_path, sizeof(config_path), "%s/sudo.conf", data.tmp_dir);
 
     char *content = NULL;
-    if (asprintf(&content, "Set developer_mode true\n"
-                           "Debug %s %s/debug.log %s\n",
+    if (asprintf(&content, "Debug %s %s/debug.log %s\n",
                  "python_plugin.so", data.tmp_dir, debug_spec) < 0)
     {
         printf("Failed to allocate string\n");
@@ -516,6 +513,7 @@ check_example_group_plugin_is_able_to_debug(void)
 
     return true;
 }
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 
 static int
 check_plugin_unload(void)
@@ -529,6 +527,7 @@ check_plugin_unload(void)
     return true;
 }
 
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 static int
 check_example_debugging(const char *debug_spec)
 {
@@ -559,6 +558,7 @@ check_example_debugging(const char *debug_spec)
     free(debug_flags_setting);
     return true;
 }
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 
 static int
 check_loading_fails(const char *name)
@@ -629,16 +629,6 @@ check_loading_fails_with_wrong_path(void)
     return check_loading_fails("wrong_path");
 }
 
-static int
-check_loading_fails_plugin_is_not_owned_by_root(void)
-{
-    sudo_conf_clear_paths();
-    VERIFY_INT(sudo_conf_read(sudo_conf_normal_mode, SUDO_CONF_ALL), true);
-
-    create_debugging_plugin_options();
-    return check_loading_fails("not_owned_by_root");
-}
-
 static int
 check_example_conversation_plugin_reason_log(int simulate_suspend, const char *description)
 {
@@ -910,10 +900,12 @@ check_example_policy_plugin_validate_invalidate(void)
 {
     const char *errstr = NULL;
 
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     // the plugin does not do any meaningful for these, so using log to validate instead
     const char *config_path = create_debug_config("py_calls@diag");
     VERIFY_NOT_NULL(config_path);
     VERIFY_INT(sudo_conf_read(config_path, SUDO_CONF_ALL), true);
+#endif
 
     create_policy_plugin_options();
 
@@ -930,7 +922,9 @@ check_example_policy_plugin_validate_invalidate(void)
 
     python_policy->close(0, 0); // no command execution
 
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     VERIFY_LOG_LINES(expected_path("check_example_policy_plugin_validate_invalidate.log"));
+#endif
     VERIFY_STR(data.stderr_str, "");
     VERIFY_STR(data.stdout_str, "");
     return true;
@@ -1545,7 +1539,9 @@ main(int argc, char *argv[])
     RUN_TEST(check_plugin_unload());
 
     RUN_TEST(check_example_group_plugin());
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     RUN_TEST(check_example_group_plugin_is_able_to_debug());
+#endif
     RUN_TEST(check_plugin_unload());
 
     RUN_TEST(check_loading_fails_with_missing_path());
@@ -1553,7 +1549,6 @@ main(int argc, char *argv[])
     RUN_TEST(check_loading_fails_with_missing_classname());
     RUN_TEST(check_loading_fails_with_wrong_classname());
     RUN_TEST(check_loading_fails_with_wrong_path());
-    RUN_TEST(check_loading_fails_plugin_is_not_owned_by_root());
     RUN_TEST(check_plugin_unload());
 
     RUN_TEST(check_example_conversation_plugin_reason_log(false, "without_suspend"));
@@ -1599,6 +1594,7 @@ main(int argc, char *argv[])
     RUN_TEST(check_python_plugins_do_not_affect_each_other());
     RUN_TEST(check_plugin_unload());
 
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
     RUN_TEST(check_example_debugging("plugin@err"));
     RUN_TEST(check_example_debugging("plugin@info"));
     RUN_TEST(check_example_debugging("load@diag"));
@@ -1609,6 +1605,7 @@ main(int argc, char *argv[])
     RUN_TEST(check_example_debugging("py_calls@info"));
     RUN_TEST(check_example_debugging("plugin@err"));
     RUN_TEST(check_plugin_unload());
+#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
 
     if (ntests != 0) {
         printf("%s: %d tests run, %d errors, %d%% success rate\n",
