@@ -692,7 +692,7 @@ matches_env_check(const char *var, bool *full_match)
 	} else {
 	    const char *val = strchr(var, '=');
 	    if (val != NULL)
-		keepit = !strpbrk(++val, "/%");
+		keepit = !strpbrk(val + 1, "/%");
 	}
     }
     debug_return_int(keepit);
@@ -919,25 +919,27 @@ rebuild_env(void)
 	}
 
 	/* Pull in vars we want to keep from the old environment. */
-	for (ep = env.old_envp; *ep; ep++) {
-	    bool keepit;
+	if (env.old_envp != NULL) {
+	    for (ep = env.old_envp; *ep; ep++) {
+		bool keepit;
 
-	    /*
-	     * Look up the variable in the env_check and env_keep lists.
-	     */
-	    keepit = env_should_keep(*ep);
+		/*
+		 * Look up the variable in the env_check and env_keep lists.
+		 */
+		keepit = env_should_keep(*ep);
 
-	    /*
-	     * Do SUDO_PS1 -> PS1 conversion.
-	     * This must happen *after* env_should_keep() is called.
-	     */
-	    if (strncmp(*ep, "SUDO_PS1=", 9) == 0)
-		ps1 = *ep + 5;
+		/*
+		 * Do SUDO_PS1 -> PS1 conversion.
+		 * This must happen *after* env_should_keep() is called.
+		 */
+		if (strncmp(*ep, "SUDO_PS1=", 9) == 0)
+		    ps1 = *ep + 5;
 
-	    if (keepit) {
-		/* Preserve variable. */
-		CHECK_PUTENV(*ep, true, false);
-		env_update_didvar(*ep, &didvar);
+		if (keepit) {
+		    /* Preserve variable. */
+		    CHECK_PUTENV(*ep, true, false);
+		    env_update_didvar(*ep, &didvar);
+		}
 	    }
 	}
 	didvar |= didvar << 16;		/* convert DID_* to KEPT_* */
@@ -999,18 +1001,20 @@ rebuild_env(void)
 	 * Copy environ entries as long as they don't match env_delete or
 	 * env_check.
 	 */
-	for (ep = env.old_envp; *ep; ep++) {
-	    /* Add variable unless it matches a blocklist. */
-	    if (!env_should_delete(*ep)) {
-		if (strncmp(*ep, "SUDO_PS1=", 9) == 0)
-		    ps1 = *ep + 5;
-		else if (strncmp(*ep, "SHELL=", 6) == 0)
-		    SET(didvar, DID_SHELL);
-		else if (strncmp(*ep, "PATH=", 5) == 0)
-		    SET(didvar, DID_PATH);
-		else if (strncmp(*ep, "TERM=", 5) == 0)
-		    SET(didvar, DID_TERM);
-		CHECK_PUTENV(*ep, true, false);
+	if (env.old_envp != NULL) {
+	    for (ep = env.old_envp; *ep; ep++) {
+		/* Add variable unless it matches a blocklist. */
+		if (!env_should_delete(*ep)) {
+		    if (strncmp(*ep, "SUDO_PS1=", 9) == 0)
+			ps1 = *ep + 5;
+		    else if (strncmp(*ep, "SHELL=", 6) == 0)
+			SET(didvar, DID_SHELL);
+		    else if (strncmp(*ep, "PATH=", 5) == 0)
+			SET(didvar, DID_PATH);
+		    else if (strncmp(*ep, "TERM=", 5) == 0)
+			SET(didvar, DID_TERM);
+		    CHECK_PUTENV(*ep, true, false);
+		}
 	    }
 	}
     }
@@ -1145,7 +1149,8 @@ bool
 validate_env_vars(char * const env_vars[])
 {
     char * const *ep;
-    char *eq, errbuf[4096];
+    char errbuf[4096];
+    char *errpos = errbuf;
     bool okvar, ret = true;
     debug_decl(validate_env_vars, SUDOERS_DEBUG_ENV);
 
@@ -1153,9 +1158,12 @@ validate_env_vars(char * const env_vars[])
 	debug_return_bool(true);	/* nothing to do */
 
     /* Add user-specified environment variables. */
-    errbuf[0] = '\0';
     for (ep = env_vars; *ep != NULL; ep++) {
-	if (def_secure_path && !user_is_exempt() &&
+	char *eq = strchr(*ep, '=');
+	if (eq == NULL || eq == *ep) {
+	    /* Must be in the form var=val. */
+	    okvar = false;
+	} else if (def_secure_path && !user_is_exempt() &&
 	    strncmp(*ep, "PATH=", 5) == 0) {
 	    okvar = false;
 	} else if (def_env_reset) {
@@ -1164,20 +1172,21 @@ validate_env_vars(char * const env_vars[])
 	    okvar = !env_should_delete(*ep);
 	}
 	if (okvar == false) {
-	    /* Not allowed, add to error string, allocating as needed. */
-	    if ((eq = strchr(*ep, '=')) != NULL)
-		*eq = '\0';
-	    if (errbuf[0] != '\0')
-		(void)strlcat(errbuf, ", ", sizeof(errbuf));
-	    if (strlcat(errbuf, *ep, sizeof(errbuf)) >= sizeof(errbuf)) {
-		errbuf[sizeof(errbuf) - 4] = '\0';
-		(void)strlcat(errbuf, "...", sizeof(errbuf));
+	    /* Not allowed, append to error buffer if space remains. */
+	    if (errpos < &errbuf[sizeof(errbuf)]) {
+		size_t varlen = strcspn(*ep, "=");
+		int len = snprintf(errpos, sizeof(errbuf) - (errpos - errbuf),
+		    "%s%.*s", errpos != errbuf ? ", " : "", (int)varlen, *ep);
+		if (len >= ssizeof(errbuf) - (errpos - errbuf)) {
+		    memcpy(&errbuf[sizeof(errbuf) - 4], "...", 4);
+		    errpos = &errbuf[sizeof(errbuf)];
+		} else {
+		    errpos += len;
+		}
 	    }
-	    if (eq != NULL)
-		*eq = '=';
 	}
     }
-    if (errbuf[0] != '\0') {
+    if (errpos != errbuf) {
 	/* XXX - audit? */
 	log_warningx(0,
 	    N_("sorry, you are not allowed to set the following environment variables: %s"), errbuf);
@@ -1269,7 +1278,7 @@ env_file_next_local(void *cookie, int *errnum)
 	val_len = strlen(++val);
 
 	/* Strip leading and trailing single/double quotes */
-	if ((val[0] == '\'' || val[0] == '\"') && val[0] == val[val_len - 1]) {
+	if ((val[0] == '\'' || val[0] == '\"') && val_len > 1 && val[0] == val[val_len - 1]) {
 	    val[val_len - 1] = '\0';
 	    val++;
 	    val_len -= 2;
