@@ -204,7 +204,7 @@ static int
 sudo_ldap_init(LDAP **ldp, const char *host, int port)
 {
     LDAP *ld;
-    int ret = LDAP_CONNECT_ERROR;
+    int ret;
     debug_decl(sudo_ldap_init, SUDOERS_DEBUG_LDAP);
 
 #ifdef HAVE_LDAPSSL_INIT
@@ -281,8 +281,10 @@ sudo_ldap_init(LDAP **ldp, const char *host, int port)
 	ret = ldap_set_option(ld, LDAP_OPT_HOST_NAME, host);
 #else
 	DPRINTF2("ldap_init(%s, %d)", host, port);
-	if ((ld = ldap_init((char *)host, port)) == NULL)
+	if ((ld = ldap_init((char *)host, port)) == NULL) {
+	    ret = LDAP_LOCAL_ERROR;
 	    goto done;
+	}
 	ret = LDAP_SUCCESS;
 #endif
     }
@@ -1874,9 +1876,9 @@ sudo_ldap_result_add_entry(struct ldap_result *lres, LDAPMessage *entry)
     last = sudo_ldap_result_last_search(lres);
     if (last != NULL) {
 	bv = sudo_ldap_get_values_len(last->ldap, entry, "sudoOrder", &rc);
-	if (rc == LDAP_NO_MEMORY) {
-	    /* XXX - return error */
-	    sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	if (bv == NULL) {
+	    if (rc == LDAP_NO_MEMORY)
+		debug_return_ptr(NULL);
 	} else {
 	    if (ldap_count_values_len(bv) > 0) {
 		/* Get the value of this attribute, 0 if not present. */
