@@ -504,9 +504,11 @@ connect_server(const char *host, const char *port, bool tls,
 	case AF_INET:
 	    addr = (char *)&((struct sockaddr_in *)res->ai_addr)->sin_addr;
 	    break;
+#ifdef HAVE_STRUCT_IN6_ADDR
 	case AF_INET6:
 	    addr = (char *)&((struct sockaddr_in6 *)res->ai_addr)->sin6_addr;
 	    break;
+#endif
 	default:
             cause = "ai_family";
 	    save_errno = EAFNOSUPPORT;
@@ -1085,7 +1087,7 @@ done:
 static bool
 fmt_initial_message(struct client_closure *closure)
 {
-    bool ret = true;
+    bool ret = false;
     debug_decl(fmt_initial_message, SUDOERS_DEBUG_UTIL);
 
     closure->state = closure->initial_state;
@@ -1116,7 +1118,6 @@ fmt_initial_message(struct client_closure *closure)
 	break;
     default:
 	sudo_warnx(U_("%s: unexpected state %d"), __func__, closure->state);
-	ret = false;
 	break;
     }
     debug_return_bool(ret);
