@@ -4,6 +4,8 @@
 # Same as test2.sh but missing the final newline.
 #
 
+: ${TESTSUDOERS=testsudoers}
+
 MYUID=`\ls -ln $TESTDIR/test2.inc | awk '{print $3}'`
 MYGID=`\ls -ln $TESTDIR/test2.inc | awk '{print $4}'`
 exec 2>&1
@@ -11,12 +13,12 @@ exec 2>&1
 echo "Testing @include without a newline"
 echo ""
 printf "@include $TESTDIR/test2.inc" | \
-    ./testsudoers -U $MYUID -G $MYGID root id
+    $TESTSUDOERS -U $MYUID -G $MYGID root id
 
 echo ""
 echo "Testing #include without a newline"
 echo ""
 printf "#include $TESTDIR/test2.inc" | \
-    ./testsudoers -U $MYUID -G $MYGID root id
+    $TESTSUDOERS -U $MYUID -G $MYGID root id
 
 exit 0
