@@ -971,8 +971,8 @@ static char *
 get_execve_info(pid_t pid, struct sudo_ptrace_regs *regs, char **pathname_out,
     int *argc_out, char ***argv_out, int *envc_out, char ***envp_out)
 {
-    char *pathname, **argv, **envp, *argbuf = NULL;
-    unsigned long path_addr, argv_addr, envp_addr;
+    char *argbuf, **argv, **envp, *pathname = NULL;
+    unsigned long argv_addr, envp_addr, path_addr;
     size_t bufsize, len, off = 0;
     int i, argc, envc = 0;
     debug_decl(get_execve_info, SUDO_DEBUG_EXEC);
@@ -992,11 +992,8 @@ get_execve_info(pid_t pid, struct sudo_ptrace_regs *regs, char **pathname_out,
 	"%s: %d: path 0x%lx, argv 0x%lx, envp 0x%lx", __func__,
 	(int)pid, path_addr, argv_addr, envp_addr);
 
-    /* Read the pathname. */
-    if (path_addr == 0) {
-	/* execve(2) will fail with EINVAL */
-	pathname = NULL;
-    } else {
+    /* Read the pathname, if not NULL. */
+    if (path_addr != 0) {
 	len = ptrace_read_string(pid, path_addr, argbuf, bufsize);
 	if (len == (size_t)-1) {
 	    sudo_debug_printf(
@@ -1004,7 +1001,7 @@ get_execve_info(pid_t pid, struct sudo_ptrace_regs *regs, char **pathname_out,
 		"unable to read execve pathname for process %d", (int)pid);
 	    goto bad;
 	}
-	pathname = argbuf;
+	/* Defer setting pathname until after all reallocations are done. */
 	off = len;
     }
 
@@ -1038,6 +1035,10 @@ get_execve_info(pid_t pid, struct sudo_ptrace_regs *regs, char **pathname_out,
 	goto bad;
     }
 
+    /* Set pathname now that argbuf has been fully allocated. */
+    if (path_addr != 0)
+	pathname = argbuf;
+
     /* Convert offsets in argv and envp to pointers. */
     argv = (char **)(argbuf + (unsigned long)argv);
     for (i = 0; i < argc; i++) {
@@ -1697,8 +1698,7 @@ ptrace_intercept_execve(pid_t pid, struct intercept_closure *closure)
     char cwd[PATH_MAX];
     unsigned long msg;
     bool ret = false;
-    struct stat sb;
-    int i;
+    int i, oldcwd = -1;
     debug_decl(ptrace_intercept_execve, SUDO_DEBUG_EXEC);
 
     /* Do not check the policy if we are executing the initial command. */
@@ -1783,16 +1783,6 @@ ptrace_intercept_execve(pid_t pid, struct intercept_closure *closure)
 	goto done;
     }
 
-    /*
-     * Short-circuit the policy check if the command doesn't exist.
-     * Otherwise, both sudo and the shell will report the error.
-     */
-    if (stat(pathname, &sb) == -1) {
-	ptrace_fail_syscall(pid, &regs, errno);
-	ret = true;
-	goto done;
-    }
-
     /* We can only pass the pathname to exececute via argv[0] (plugin API). */
     argv[0] = pathname;
     if (argc == 0) {
@@ -1806,8 +1796,9 @@ ptrace_intercept_execve(pid_t pid, struct intercept_closure *closure)
     sudo_debug_printf(SUDO_DEBUG_INFO, "%s: %d: checking policy for %s",
 	__func__, (int)pid, pathname);
     if (!intercept_check_policy(pathname, argc, argv, envc, envp, cwd,
-	    closure)) {
-	sudo_warnx("%s", U_(closure->errstr));
+	    &oldcwd, closure)) {
+	if (closure->errstr != NULL)
+	    sudo_warnx("%s", U_(closure->errstr));
     }
 
     switch (closure->state) {
@@ -1930,15 +1921,25 @@ ptrace_intercept_execve(pid_t pid, struct intercept_closure *closure)
 	    }
 	}
 	break;
-    default:
+    case POLICY_REJECT:
 	/* If rejected, fake the syscall and set return to EACCES */
-	ptrace_fail_syscall(pid, &regs, EACCES);
+	errno = EACCES;
+	FALLTHROUGH;
+    default:
+	ptrace_fail_syscall(pid, &regs, errno);
 	break;
     }
 
     ret = true;
 
 done:
+    if (oldcwd != -1) {
+        if (fchdir(oldcwd) == -1) {
+            sudo_debug_printf(SUDO_DEBUG_ERROR|SUDO_DEBUG_ERRNO,
+                "%s: unable to restore saved cwd", __func__);
+        }
+        close(oldcwd);
+    }
     free(buf);
     intercept_closure_reset(closure);
 
