@@ -34,114 +34,57 @@
 #include "sudo_plugin_int.h"
 #include "sudo_dso.h"
 
-/* We always use the same name for the sudoers plugin, regardless of the OS */
-#define SUDOERS_PLUGIN	"sudoers.so"
-
 #ifdef ENABLE_SUDO_PLUGIN_API
-static int
-sudo_stat_plugin(struct plugin_info *info, char *fullpath,
-    size_t pathsize, struct stat *sb)
+static bool
+sudo_qualify_plugin(struct plugin_info *info, char *fullpath, size_t pathsize)
 {
-    int status = -1;
-    size_t len;
+    const char *plugin_dir = sudo_conf_plugin_dir_path();
+    int len;
     debug_decl(sudo_stat_plugin, SUDO_DEBUG_PLUGIN);
 
     if (info->path[0] == '/') {
 	if (strlcpy(fullpath, info->path, pathsize) >= pathsize) {
 	    errno = ENAMETOOLONG;
-	    goto done;
+	    goto bad;
 	}
-	status = stat(fullpath, sb);
     } else {
 #ifdef STATIC_SUDOERS_PLUGIN
 	/* Check static symbols. */
-	if (strcmp(info->path, SUDOERS_PLUGIN) == 0) {
+	if (strcmp(info->path, _PATH_SUDOERS_PLUGIN) == 0) {
 	    if (strlcpy(fullpath, info->path, pathsize) >= pathsize) {
 		errno = ENAMETOOLONG;
-		goto done;
+		goto bad;
 	    }
-	    /* Plugin is static, fake up struct stat. */
-	    memset(sb, 0, sizeof(*sb));
-	    sb->st_uid = ROOT_UID;
-	    sb->st_mode = S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH;
-	    status = 0;
-	    goto done;
+	    /* Plugin is static, do not fully-qualify. */
+	    debug_return_bool(true);
 	}
 #endif /* STATIC_SUDOERS_PLUGIN */
 
-	if (sudo_conf_plugin_dir_path() == NULL) {
+	if (plugin_dir == NULL) {
 	    errno = ENOENT;
-	    goto done;
+	    goto bad;
 	}
-
-	len = snprintf(fullpath, pathsize, "%s%s", sudo_conf_plugin_dir_path(),
-	    info->path);
-	if (len >= pathsize) {
+	len = snprintf(fullpath, pathsize, "%s%s", plugin_dir, info->path);
+	if (len < 0 || (size_t)len >= pathsize) {
 	    errno = ENAMETOOLONG;
-	    goto done;
-	}
-	status = stat(fullpath, sb);
-    }
-    if (status == -1) {
-	char *newpath = sudo_stat_multiarch(fullpath, sb);
-	if (newpath != NULL) {
-	    len = strlcpy(fullpath, newpath, pathsize);
-	    free(newpath);
-	    if (len >= pathsize) {
-		errno = ENAMETOOLONG;
-		goto done;
-	    }
-	    status = 0;
+	    goto bad;
 	}
     }
-done:
-    debug_return_int(status);
-}
-
-static bool
-sudo_check_plugin(struct plugin_info *info, char *fullpath, size_t pathsize)
-{
-    struct stat sb;
-    bool ret = false;
-    debug_decl(sudo_check_plugin, SUDO_DEBUG_PLUGIN);
-
-    if (sudo_stat_plugin(info, fullpath, pathsize, &sb) != 0) {
-	sudo_warnx(U_("error in %s, line %d while loading plugin \"%s\""),
-	    _PATH_SUDO_CONF, info->lineno, info->symbol_name);
-	if (info->path[0] == '/') {
-	    sudo_warn("%s", info->path);
-	} else {
-	    sudo_warn("%s%s",
-		sudo_conf_plugin_dir_path() ? sudo_conf_plugin_dir_path() : "",
-		info->path);
-	}
-	goto done;
-    }
-
-    if (!sudo_conf_developer_mode()) {
-        if (sb.st_uid != ROOT_UID) {
-            sudo_warnx(U_("error in %s, line %d while loading plugin \"%s\""),
-                _PATH_SUDO_CONF, info->lineno, info->symbol_name);
-            sudo_warnx(U_("%s must be owned by uid %d"), fullpath, ROOT_UID);
-            goto done;
-        }
-        if ((sb.st_mode & (S_IWGRP|S_IWOTH)) != 0) {
-            sudo_warnx(U_("error in %s, line %d while loading plugin \"%s\""),
-                _PATH_SUDO_CONF, info->lineno, info->symbol_name);
-            sudo_warnx(U_("%s must be only be writable by owner"), fullpath);
-            goto done;
-        }
-    }
-    ret = true;
-
-done:
-    debug_return_bool(ret);
+    debug_return_bool(true);
+bad:
+    sudo_warnx(U_("error in %s, line %d while loading plugin \"%s\""),
+	_PATH_SUDO_CONF, info->lineno, info->symbol_name);
+    if (info->path[0] != '/' && plugin_dir != NULL)
+	sudo_warn("%s%s", plugin_dir, info->path);
+    else
+	sudo_warn("%s", info->path);
+    debug_return_bool(false);
 }
 #else
 static bool
-sudo_check_plugin(struct plugin_info *info, char *fullpath, size_t pathsize)
+sudo_qualify_plugin(struct plugin_info *info, char *fullpath, size_t pathsize)
 {
-    debug_decl(sudo_check_plugin, SUDO_DEBUG_PLUGIN);
+    debug_decl(sudo_qualify_plugin, SUDO_DEBUG_PLUGIN);
     (void)strlcpy(fullpath, info->path, pathsize);
     debug_return_bool(true);
 }
@@ -267,8 +210,8 @@ sudo_load_plugin(struct plugin_info *info, bool quiet)
     bool ret = false;
     debug_decl(sudo_load_plugin, SUDO_DEBUG_PLUGIN);
 
-    /* Check plugin owner/mode and fill in path */
-    if (!sudo_check_plugin(info, path, sizeof(path)))
+    /* Fill in path from info and plugin dir. */
+    if (!sudo_qualify_plugin(info, path, sizeof(path)))
 	goto done;
 
     /* Open plugin and map in symbol */
@@ -452,7 +395,7 @@ sudo_load_sudoers_plugin(const char *symbol_name, bool optional)
 	goto done;
     }
     info->symbol_name = strdup(symbol_name);
-    info->path = strdup(SUDOERS_PLUGIN);
+    info->path = strdup(_PATH_SUDOERS_PLUGIN);
     if (info->symbol_name == NULL || info->path == NULL) {
 	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
 	free_plugin_info(info);
