@@ -64,7 +64,7 @@ static struct sudo_preload_symbol sudo_sudoers_plugin_symbols[] = {
  */
 static struct sudo_preload_table sudo_preload_table[] = {
     { (char *)0, SUDO_DSO_DEFAULT, sudo_rtld_default_symbols },
-    { "sudoers.so", &sudo_sudoers_plugin_symbols, sudo_sudoers_plugin_symbols },
+    { _PATH_SUDOERS_PLUGIN, &sudo_sudoers_plugin_symbols, sudo_sudoers_plugin_symbols },
     { (char *)0, (void *)0, (struct sudo_preload_symbol *)0 }
 };
 
